###########################
## Visualize tissue data ##
###########################

#' Visualize tissue data
#'
#' Visualize the expression of protein of interest in each target tissue by cell
#' types.
#'
#' @param data Input the list object generated by \code{hpa_download()} or
#'   \code{hpa_subset()}. Require the \code{normal_tissue} dataset.
#' @param targetGene Vector of strings of HGNC gene symbols.
#' @param targetTissue Vector of strings of normal tissues.
#' @param targetCellType Vector of strings of normal cell types.
#' @param color Vector of 4 colors used to depict different expression levels.
#' @param customTheme Logical argument. If \code{TRUE}, the function will return
#'   a barebone ggplot2 plot to be customized further.
#'
#' @return This function will return a ggplot2 plot object, which can be further
#'   modified if desirable. The tissue data is visualized as a heatmap: x axis
#'   contains inquired protein and y axis contains tissue/cells of interest.
#'   
#' @family visualization functions
#'
#' @examples
#'   data("hpa_downloaded_histology_v18")
#'   geneList <- c('TP53', 'EGFR', 'CD44', 'PTEN', 'IDH1', 'IDH2', 'CYCS')
#'   tissueList <- c('breast', 'cerebellum', 'skin 1')
#'
#'   ## A typical function call
#'   hpaVisTissue(data=hpa_downloaded_histology_v18,
#'                targetGene=geneList,
#'                targetTissue=tissueList)
#'
#' @import dplyr
#' @import ggplot2
#' @export

hpaVisTissue <- function(data, 
                         targetGene, 
                         targetTissue, 
                         targetCellType=NULL,
                         color=c('#ffffb2', '#fecc5c', '#fd8d3c', '#e31a1c'),
                         customTheme=FALSE) {
    
    plotData <- data$normal_tissue %>%
        filter(gene %in% targetGene) %>%
        filter(tissue %in% targetTissue)
    
    if(!is.null(targetCellType)) {
        plotData <- filter(plotData, cell_type %in% targetCellType)
    }
    
    plotData <- mutate(plotData,
                       tissue_cell=paste0(tissue, ' / ', cell_type),
                       level=factor(level, 
                                    levels=c('High', 'Medium', 
                                             'Low', 'Not detected')))
    
    levelColors <- c('Not detected'=color[1],
                     'Low'=color[2],
                     'Medium'=color[3],
                     'High'=color[4])
    
    plot <- ggplot(plotData, aes(x=gene, y=tissue_cell)) +
        geom_tile(aes(fill=level)) +
        scale_x_discrete(limits=targetGene) +
        scale_fill_manual(values=levelColors)
    
    if(!customTheme) {
        plot <- plot + 
            ylab('Tissue / Cell') +
            xlab('Genes') +
            theme(axis.text.x=element_text(angle=90, hjust=1))
    }
    
    return(plot)       
}



##############################
## Visualize pathology data ##
##############################

#' Visualize pathology data
#'
#' Visualize the expression of genes of interest in each cancer.
#'
#' @param data Input the list object generated by \code{hpa_download()} or
#'   \code{hpa_subset()}. Require the \code{pathology} dataset.
#' @param targetGene Vector of strings of HGNC gene symbols.
#' @param targetCancer Vector of strings of normal tissues. The function will
#'   plot all available cancer by default.
#' @param color Vector of 4 colors used to depict different expression levels.
#' @param customTheme Logical argument. If \code{TRUE}, the function will return
#'   a barebone ggplot2 plot to be customized further.
#'
#' @return This function will return a ggplot2 plot object, which can be further
#'   modified if desirable. The pathology data is visualized as multiple bar
#'   graphs, one for each type of cancer. For each bar graph, x axis contains
#'   the inquired protein and y axis contains the proportion of patients.
#'   
#' @family visualization functions
#'
#' @examples
#'   data("hpa_downloaded_histology_v18")
#'   geneList <- c('TP53', 'EGFR', 'CD44', 'PTEN', 'IDH1', 'IDH2', 'CYCS')
#'   cancerList <- c('breast cancer', 'glioma', 'melanoma')
#'
#'   ## A typical function call
#'   hpaVisPatho(data=hpa_downloaded_histology_v18,
#'                  targetGene=geneList)
#'
#' @import dplyr
#' @import ggplot2
#' @export

hpaVisPatho <- function(data, 
                        targetGene, 
                        targetCancer=NULL, 
                        color=c('#ffffb2', '#fecc5c', '#fd8d3c', '#e31a1c'),
                        customTheme=FALSE) {
    
    plotData <- data$pathology %>%
        filter(gene %in% targetGene)
    
    if(!is.null(targetCancer)) {
        plotData <- filter(plotData, cancer %in% targetCancer)
    }
    
    plotData <- plotData %>%
        select(gene, cancer, high, medium, low, not_detected) %>%
        rename('High'='high', 'Medium'='medium', 
               'Low'='low', 'Not detected'='not_detected') %>%
        reshape2::melt(measure.vars=c('High', 'Medium', 'Low', 'Not detected'),
                       variable.name='level',
                       value.name='patient_count')      
    
    levelColors <- c('Not detected'=color[1],
                     'Low'=color[2],
                     'Medium'=color[3],
                     'High'=color[4])
    
    plot <- ggplot(plotData, aes(x=gene, y=patient_count, fill=level)) +
        geom_bar(stat='identity', position='fill') +
        scale_x_discrete(limits=targetGene) +
        scale_fill_manual(values=levelColors) +
        facet_wrap(~ cancer)
    
    if(!customTheme) {
        plot <- plot + 
            ylab('Patient proportions') +
            xlab('Genes') +
            theme(axis.text.x=element_text(angle=90, hjust=1))
    }
    
    return(plot)       
}


############################
## Visualize subcell data ##
############################

#' Visualize subcellular location data
#'
#' Visualize the the confirmed subcellular locations of genes of interest.
#'
#' @param data Input the list object generated by \code{hpa_download()} or
#'   \code{hpa_subset()}. Require the \code{subcellular_location} dataset.
#' @param targetGene Vector of strings of HGNC gene symbols.
#' @param color Vector of 2 colors used to depict if the protein expresses in a
#'   location or not.
#' @param customTheme Logical argument. If \code{TRUE}, the function will return
#'   a barebone ggplot2 plot to be customized further.
#'
#' @return This function will return a ggplot2 plot object, which can be further
#'   modified if desirable. The subcellular location data is visualized as a
#'   tile graph, in which the x axis includes the inquired proteins and the y
#'   axis contain the subcellular locations.
#'   
#' @family visualization functions
#'
#' @examples
#'   data("hpa_downloaded_histology_v18")
#'   geneList <- c('TP53', 'EGFR', 'CD44', 'PTEN', 'IDH1', 'IDH2', 'CYCS')
#'
#'   ## A typical function call
#'   hpaVisSubcell(data=hpa_downloaded_histology_v18,
#'                   targetGene=geneList)
#'
#' @import dplyr
#' @import ggplot2
#' @importFrom tidyr unnest
#' @importFrom tibble as_tibble
#' @export

hpaVisSubcell <- function(data, 
                          targetGene, 
                          color=c('#ffffb2', '#e31a1c'),
                          customTheme=FALSE) {
    
    plotData <- data$subcellular_location %>%
        filter(gene %in% targetGene) %>%
        mutate(sub_location=strsplit(go_id, ';')) %>%
        tidyr::unnest(sub_location) %>%
        select(sub_location, gene) %>%
        table() %>%
        as_tibble() %>%
        mutate(n=factor(n, levels=c('0', '1')))
    
    levelColors <- c('0'=color[1],
                     '1'=color[length(color)])
    
    plot <- ggplot(plotData, aes(x=gene, y=sub_location)) +
        geom_tile(aes(fill=n), colour="grey50") +
        scale_x_discrete(limits=targetGene) +
        scale_fill_manual(values=levelColors)
    
    if(!customTheme) {
        plot <- plot + 
            ylab('Subcellular locations') +
            xlab('Genes') +
            theme(axis.text.x=element_text(angle=90, hjust=1))  +
            theme(legend.position="none")
    }
    
    return(plot)       
}


